// Copyright 2023 Azul Systems, Inc.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

package sun.nio.ch.sctp;

import jdk.internal.crac.OpenResourcePolicies;
import jdk.internal.crac.JDKSocketResourceBase;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.nio.channels.spi.AbstractSelectableChannel;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Predicate;

abstract class SctpResource extends JDKSocketResourceBase {
    private Set<InetSocketAddress> localCopy;
    private Set<SocketAddress> remoteCopy;

    public SctpResource(AbstractSelectableChannel sctpChannel) {
        super(sctpChannel);
    }

    @Override
    protected OpenResourcePolicies.Policy findPolicy(boolean isRestore) {
        if (!isRestore) {
            localCopy = Set.copyOf(getLocalAddresses());
            remoteCopy = Set.copyOf(getRemoteAddresses());
        }
        List<Predicate<Map<String, String>>> localMatchers = localCopy.stream()
                .map(addr -> getMatcher(addr, "localAddress", "localPort", null)).toList();
        List<Predicate<Map<String, String>>> remoteMatchers = remoteCopy.stream()
                .map(addr -> getMatcher(addr, "remoteAddress", "remotePort", null)).toList();
        return OpenResourcePolicies.find(false, OpenResourcePolicies.SOCKET,
                params -> localMatchers.stream().anyMatch(matcher -> matcher.test(params)) &&
                        remoteMatchers.stream().anyMatch(matcher -> matcher.test(params)));
    }

    @Override
    protected void reset() {
        localCopy = null;
        remoteCopy = null;
    }

    @Override
    protected abstract void closeBeforeCheckpoint() throws IOException;

    protected abstract Set<SocketAddress> getRemoteAddresses();

    protected abstract HashSet<InetSocketAddress> getLocalAddresses();
}
